SystemOrganization addCategory: #'Mason-Core-Build'!
SystemOrganization addCategory: #'Mason-Core-Products'!
SystemOrganization addCategory: #'Mason-Core-Targets'!
SystemOrganization addCategory: #'Mason-Core-Utilities'!

Object subclass: #MDBuild
	instanceVariableNames: 'jobs products process queue done'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Build'!

!MDBuild class methodsFor: 'as yet unclassified' stamp: 'cwp 1/5/2009 21:50'!
run: aBlock 
	(MDTarget with: aBlock) createBuild runAndWait! !

!MDBuild methodsFor: 'initialization' stamp: 'cwp 12/14/2008 16:22'!
addJob: aJob
	jobs add: aJob.
	! !

!MDBuild methodsFor: 'accessing' stamp: 'cwp 12/22/2008 20:42'!
allProducts
	^ jobs
		inject: Set new
		into: 
			[:all :ea | 
			all addAll: ea supplies.
			all]! !

!MDBuild methodsFor: 'accessing' stamp: 'cwp 12/22/2008 20:43'!
allRequirements
	^ jobs
		inject: Set new
		into: 
			[:all :ea | 
			all addAll: ea requires.
			all]! !

!MDBuild methodsFor: 'running' stamp: 'cwp 12/16/2008 09:13'!
canRunJob: job 
	^ job canRun and: [ job requires allSatisfy: [ :ea | products includes: ea ] ]! !

!MDBuild methodsFor: 'running' stamp: 'cwp 12/22/2008 21:43'!
delete: aPath
	| parent basename |
	parent := FileDirectory on: aPath parent asString.
	basename := aPath basename.
	(parent directoryExists: basename) 
		ifTrue: [(FileDirectory on: aPath asString) recursiveDelete]
		ifFalse:
			[(parent fileExists: basename)
				ifTrue: [parent deleteFileNamed: basename]]! !

!MDBuild methodsFor: 'initialization' stamp: 'cwp 9/16/2009 11:25'!
initialize
	jobs := Set new.
	queue := SharedQueue new! !

!MDBuild methodsFor: 'accessing' stamp: 'cwp 12/16/2008 07:42'!
name
	^ 'build-' , self identityHash printString! !

!MDBuild methodsFor: 'running' stamp: 'cwp 9/16/2009 23:40'!
prepareToRun
	| internal external |
	internal := self allProducts.
	external := self allRequirements difference: internal.
	products := Set new.
	external do:
		[:reference | 
		reference exists
			ifTrue: [products add: reference]
			ifFalse: [self error: 'Missing requirement: ', reference asString]]! !

!MDBuild methodsFor: 'running' stamp: 'cwp 12/16/2008 23:02'!
recognize: aJob
	products addAll: aJob supplies.
	self scanJobs.! !

!MDBuild methodsFor: 'running' stamp: 'cwp 12/19/2008 03:18'!
run
	self runSignalling: Semaphore new! !

!MDBuild methodsFor: 'running' stamp: 'cwp 12/16/2008 08:56'!
run: aJob
	| message |
	aJob 
		runIn: self 
		signalWith: 
			[message := Message selector: #recognize: argument: aJob.
			queue nextPut: message]! !

!MDBuild methodsFor: 'running' stamp: 'cwp 1/5/2009 21:50'!
runAndWait
	| sem |
	sem := Semaphore new.
	self runSignalling: sem.
	sem wait.! !

!MDBuild methodsFor: 'running' stamp: 'cwp 12/16/2008 23:04'!
runLoop
	| message |
	done := false.
	[done] whileFalse:
		[message := queue next.
		message sendTo: self]! !

!MDBuild methodsFor: 'running' stamp: 'cwp 1/5/2009 21:49'!
runSignalling: aSemaphore 
	self prepareToRun.
	self scanJobs.
	process := 
		[self runLoop.
		aSemaphore signal] 
			forkAt: Processor userBackgroundPriority
			named: self name! !

!MDBuild methodsFor: 'running' stamp: 'cwp 12/22/2008 16:13'!
scanJobs
	done := true.
	jobs do: 
		[ :job | 
		job isComplete ifFalse: 
			[ done := false.
			(self canRunJob: job) ifTrue: [ self run: job ] ] ].
	done ifTrue: [ jobs do: [ :ea | ea done ] ]! !

!MDBuild methodsFor: 'running' stamp: 'cwp 12/16/2008 09:08'!
schedule: aJob
	| message |
	message := Message selector: #run: argument: aJob.
	queue nextPut: message.! !

!MDBuild methodsFor: 'accessing' stamp: 'cwp 9/20/2009 11:18'!
sortedProducts
	^ self allProducts asArray sort: [:a : b | a pathString <= b pathString]! !

!MDBuild methodsFor: 'running' stamp: 'cwp 12/16/2008 23:59'!
stop
	jobs do: [:ea | ea stop].
	done := true.
	queue nextPut: (Message selector: #yourself)! !

Object subclass: #MDBuildContext
	instanceVariableNames: 'build root current temp notifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

!MDBuildContext methodsFor: 'accessing' stamp: 'cwp 9/21/2009 17:53'!
addJob: aJob
	build addJob: aJob.
	aJob supplies do: (notifier ifNil: [[:ea]])! !

!MDBuildContext methodsFor: 'accessing' stamp: 'cwp 9/20/2009 22:09'!
allProducts
	^ build allProducts! !

!MDBuildContext methodsFor: 'accessing' stamp: 'cwp 9/16/2009 22:04'!
build
	^ build! !

!MDBuildContext methodsFor: 'initialization' stamp: 'cwp 9/16/2009 11:23'!
chooseRootDirectory
	| name count ref |
	name := build name.
	count := 1.
	ref := FSReference defaultDirectory / name , count asString.
	[ ref exists ] whileTrue: 
		[ count := count + 1.
		ref := ref , count asString ].
	^ ref! !

!MDBuildContext methodsFor: 'private' stamp: 'cwp 9/20/2009 11:24'!
createAllParentDirectories
	[self createParentDirectories] whileFalse! !

!MDBuildContext methodsFor: 'private' stamp: 'cwp 9/16/2009 22:51'!
createDirectory: aReference
	MDCreateDirectoryJob new
		path: aReference;
		addToBuild: build! !

!MDBuildContext methodsFor: 'private' stamp: 'cwp 9/20/2009 11:51'!
createParentDirectories
	| intermediate artifacts parent |
	intermediate := Set new.
	artifacts := build allProducts.
	artifacts do: 
		[ :ref | 
		ref = root ifFalse: 
			[ parent := ref parent.
			(artifacts includes: parent) ifFalse: [ intermediate add: parent ] ] ].
	intermediate do: [ :ea | self createDirectory: ea ].
	^ intermediate isEmpty! !

!MDBuildContext methodsFor: 'paths' stamp: 'cwp 9/20/2009 23:12'!
directory
	^ current! !

!MDBuildContext methodsFor: 'paths' stamp: 'cwp 9/20/2009 09:53'!
directory: anObject do: aBlock
	| pCurrent pTemp |
	pCurrent := current.
	pTemp := temp.
	[current := current resolve: anObject.
	temp := temp resolve: anObject.
	aBlock value] ensure:
		[current := pCurrent.
		temp := pTemp]! !

!MDBuildContext methodsFor: 'paths' stamp: 'cwp 9/21/2009 15:24'!
filesystem: aFilesystem named: aString do: aBlock
	| pCurrent pTemp |
	pCurrent := current.
	pTemp := temp.
	[current := aFilesystem root.
	temp := temp resolve: aString.
	aBlock value] ensure:
		[current := pCurrent.
		temp := pTemp]! !

!MDBuildContext methodsFor: 'paths' stamp: 'cwp 9/21/2009 17:50'!
filesystem: aFilesystem named: aString do: aBlock notify: notifyBlock

	| pCurrent pTemp |
	pCurrent := current.
	pTemp := temp.
	[current := aFilesystem root.
	temp := temp resolve: aString.
	notifier := notifyBlock.
	aBlock value] ensure:
		[current := pCurrent.
		temp := pTemp]! !

!MDBuildContext methodsFor: 'accessing' stamp: 'cwp 9/20/2009 12:07'!
getJobsFrom: aTarget 
	aTarget addAllJobsTo: self.
	self createAllParentDirectories! !

!MDBuildContext methodsFor: 'paths' stamp: 'cwp 9/20/2009 09:54'!
image: aString do: aBlock
	| previous |
	previous := temp.
	[temp := previous / (aString, '.image').
	aBlock value]
		ensure: [temp := previous]! !

!MDBuildContext methodsFor: 'initialization' stamp: 'cwp 9/20/2009 22:59'!
initialize
	build := MDBuild new.
	root := self chooseRootDirectory.
	current := root.
	temp := root resolve: '~tmp'! !

!MDBuildContext methodsFor: 'paths' stamp: 'cwp 9/16/2009 18:41'!
resolve: anObject
	^ self directory resolve: anObject! !

!MDBuildContext methodsFor: 'paths' stamp: 'cwp 9/20/2009 23:12'!
temp
	^ temp! !

!MDBuildContext methodsFor: 'paths' stamp: 'cwp 9/16/2009 18:43'!
temp: anObject
	^ self temp resolve: anObject! !

!MDBuildContext methodsFor: 'paths' stamp: 'cwp 9/20/2009 23:06'!
zip: anObject do: aBlock
	| pCurrent pTemp |
	pCurrent := current.
	pTemp := temp.
	[current := (FSZipFilesystem atReference: (current resolve: anObject)) root.
	temp := temp resolve: anObject.
	aBlock value] ensure:
		[current := pCurrent.
		temp := pTemp]! !

Object subclass: #MDJob
	instanceVariableNames: 'state process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Build'!

MDJob subclass: #MDCreateDirectoryJob
	instanceVariableNames: 'path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Build'!

!MDCreateDirectoryJob methodsFor: 'accessing' stamp: 'cwp 12/14/2008 18:03'!
path
	^ path! !

!MDCreateDirectoryJob methodsFor: 'accessing' stamp: 'cwp 9/16/2009 23:24'!
path: anObject
	path := anObject! !

!MDCreateDirectoryJob methodsFor: 'as yet unclassified' stamp: 'cwp 9/20/2009 12:34'!
requires
	^ Set with: path parent! !

!MDCreateDirectoryJob methodsFor: 'running' stamp: 'cwp 1/13/2009 22:15'!
runIn: aBuild
	path asReference ensureDirectory! !

!MDCreateDirectoryJob methodsFor: 'dependencies' stamp: 'cwp 12/14/2008 18:04'!
supplies
	^ Set with: path! !

MDJob subclass: #MDCreateZipJob
	instanceVariableNames: 'zip requires reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Build'!

!MDCreateZipJob methodsFor: 'running' stamp: 'cwp 9/20/2009 23:22'!
addToBuild: aBuild 
	requires := aBuild allProducts select: [ :ea | reference contains: ea ].
	super addToBuild: aBuild! !

!MDCreateZipJob methodsFor: 'accessing' stamp: 'cwp 9/20/2009 23:23'!
reference
	^ reference! !

!MDCreateZipJob methodsFor: 'accessing' stamp: 'cwp 9/20/2009 23:23'!
reference: anObject 
	reference := anObject! !

!MDCreateZipJob methodsFor: 'dependencies' stamp: 'cwp 9/20/2009 23:17'!
requires
	^ requires! !

!MDCreateZipJob methodsFor: 'running' stamp: 'cwp 9/20/2009 23:22'!
run
	zip
		writeToFileNamed: reference pathString;
		close! !

!MDCreateZipJob methodsFor: 'dependencies' stamp: 'cwp 9/20/2009 23:22'!
supplies
	^ Set with: reference! !

!MDCreateZipJob methodsFor: 'accessing' stamp: 'cwp 9/20/2009 23:17'!
zip
	^ zip! !

!MDCreateZipJob methodsFor: 'accessing' stamp: 'cwp 9/20/2009 23:17'!
zip: anObject
	zip := anObject! !

MDJob subclass: #MDFileJob
	instanceVariableNames: 'source dest'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Build'!

MDFileJob subclass: #MDCopyFileJob
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Build'!

!MDCopyFileJob methodsFor: 'as yet unclassified' stamp: 'cwp 7/23/2009 23:57'!
run
	source asReference copyTo: dest asReference! !

!MDFileJob class methodsFor: 'as yet unclassified' stamp: 'cwp 1/10/2009 23:18'!
source: sourcePath dest: destPath
	^ self new setSource: sourcePath dest: destPath! !

!MDFileJob methodsFor: 'dependencies' stamp: 'cwp 1/10/2009 22:52'!
requires
	^ Set 
		with: dest parent 
		with: source! !

!MDFileJob methodsFor: 'initialize-release' stamp: 'cwp 1/10/2009 22:52'!
setSource: sourcePath dest: destPath
	source := sourcePath.
	dest := destPath! !

!MDFileJob methodsFor: 'dependencies' stamp: 'cwp 1/10/2009 22:53'!
supplies
	^ Set with: dest! !

MDJob subclass: #MDFilesystemJob
	instanceVariableNames: 'filesystem requires supplies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Build'!

MDFilesystemJob subclass: #MDCloseFilesystemJob
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Build'!

!MDCloseFilesystemJob methodsFor: 'dependencies' stamp: 'cwp 9/21/2009 17:44'!
run
	filesystem close! !

!MDFilesystemJob methodsFor: 'accessing' stamp: 'cwp 9/21/2009 17:06'!
filesystem
	^ filesystem! !

!MDFilesystemJob methodsFor: 'accessing' stamp: 'cwp 9/21/2009 17:06'!
filesystem: anObject
	filesystem := anObject! !

!MDFilesystemJob methodsFor: 'accessing' stamp: 'cwp 9/21/2009 17:07'!
requires
	^ requires! !

!MDFilesystemJob methodsFor: 'accessing' stamp: 'cwp 9/21/2009 17:07'!
requires: anObject
	requires := anObject! !

!MDFilesystemJob methodsFor: 'accessing' stamp: 'cwp 9/21/2009 17:07'!
supplies
	^ supplies! !

!MDFilesystemJob methodsFor: 'accessing' stamp: 'cwp 9/21/2009 17:07'!
supplies: aSet
	supplies := aSet! !

MDFilesystemJob subclass: #MDOpenFilesystemJob
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Build'!

!MDOpenFilesystemJob methodsFor: 'accessing' stamp: 'cwp 9/21/2009 17:44'!
run
	filesystem open! !

!MDJob methodsFor: 'running' stamp: 'cwp 12/14/2008 23:23'!
addToBuild: aBuild
	aBuild addJob: self! !

!MDJob methodsFor: 'running' stamp: 'cwp 12/16/2008 23:45'!
canRun
	^ state = #notYetRunning! !

!MDJob methodsFor: 'running' stamp: 'cwp 12/16/2008 23:43'!
done
	process := nil.
	state := #notYetRunning! !

!MDJob methodsFor: 'initialize-release' stamp: 'cwp 12/16/2008 23:40'!
initialize
	state := #notYetRunning! !

!MDJob methodsFor: 'running' stamp: 'cwp 12/16/2008 23:45'!
isComplete
	^ state = #complete! !

!MDJob methodsFor: 'dependencies' stamp: 'cwp 12/14/2008 16:31'!
requires
	^ Set new! !

!MDJob methodsFor: 'running' stamp: 'cwp 1/10/2009 13:18'!
run
	self subclassResponsibility! !

!MDJob methodsFor: 'running' stamp: 'cwp 12/16/2008 08:57'!
runIn: aBuild
	self run! !

!MDJob methodsFor: 'running' stamp: 'cwp 12/22/2008 19:08'!
runIn: aBuild signalWith: aBlock
	state = #notYetRunning ifFalse: [self error: 'Job is already running'].
	state := #running.
	process := 
		[self runIn: aBuild.
		state := #complete.
		aBlock value]
			forkAt: Processor userBackgroundPriority - 1
			named: self name.! !

!MDJob methodsFor: 'running' stamp: 'cwp 12/16/2008 23:55'!
stop
	process ifNotNil:
		[process terminate.
		process := nil.
		state := #interrupted].
	self canRun ifTrue: [state := #cancelled]! !

!MDJob methodsFor: 'dependencies' stamp: 'cwp 12/14/2008 16:32'!
supplies
	^ Set new! !

MDJob subclass: #MDRunScriptJob
	instanceVariableNames: 'scripts targetImage baseImage loadScript'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Build'!

!MDRunScriptJob methodsFor: 'accessing' stamp: 'cwp 12/13/2008 23:25'!
addScript: aPath
	scripts add: aPath! !

!MDRunScriptJob methodsFor: 'accessing' stamp: 'cwp 12/19/2008 13:37'!
baseImagePath
	^ baseImage ! !

!MDRunScriptJob methodsFor: 'accessing' stamp: 'cwp 12/19/2008 13:37'!
baseImagePath: aPath
	baseImage := aPath! !

!MDRunScriptJob methodsFor: 'accessing' stamp: 'cwp 9/17/2009 22:21'!
changesPath
	^ self targetImagePath asPath , 'changes'! !

!MDRunScriptJob methodsFor: 'accessing' stamp: 'cwp 9/17/2009 22:22'!
imagePath
	^ self targetImagePath asPath , 'image'! !

!MDRunScriptJob methodsFor: 'intialize-release' stamp: 'cwp 12/14/2008 11:39'!
initialize
	super initialize.
	scripts := OrderedCollection new.! !

!MDRunScriptJob methodsFor: 'accessing' stamp: 'cwp 9/17/2009 09:43'!
loadScript
	^ loadScript! !

!MDRunScriptJob methodsFor: 'accessing' stamp: 'cwp 9/17/2009 09:43'!
loadScript: anObject
	loadScript := anObject! !

!MDRunScriptJob methodsFor: 'accessing' stamp: 'cwp 12/20/2008 12:09'!
name
	^ self imagePath basename! !

!MDRunScriptJob methodsFor: 'dependencies' stamp: 'cwp 12/19/2008 13:40'!
requires
	^ Set new
		add: self baseImagePath;
		add: self targetImagePath parent;
		addAll: scripts;
		yourself! !

!MDRunScriptJob methodsFor: 'running' stamp: 'cwp 9/17/2009 09:45'!
runIn: aBuild 
	self writeLoadScript: loadScript.
	self runLoadScript: loadScript! !

!MDRunScriptJob methodsFor: 'running' stamp: 'cwp 9/17/2009 22:11'!
runLoadScript: aReference 
	| arguments proc d |
	d := Delay forMilliseconds: 50.
	arguments := Array 
		with: self baseImagePath pathString
		with: aReference pathString.
	proc := OSProcess thisOSProcess 
		forkJob: OSProcess programName
		arguments: arguments
		environment: nil
		descriptors: nil.
	proc ifNil: [ OSProcess noAccessorAvailable ].
	[ proc runState == #complete ] whileFalse: [ d wait ]! !

!MDRunScriptJob methodsFor: 'dependencies' stamp: 'cwp 12/15/2008 14:08'!
supplies
	^ Set 
		with: targetImage
		with: targetImage , 'changes'! !

!MDRunScriptJob methodsFor: 'accessing' stamp: 'cwp 12/15/2008 14:08'!
targetImagePath
	^ targetImage! !

!MDRunScriptJob methodsFor: 'accessing' stamp: 'cwp 9/16/2009 23:30'!
targetImagePath: anObject 
	targetImage := anObject ! !

!MDRunScriptJob methodsFor: 'running' stamp: 'cwp 9/17/2009 22:34'!
writeFilein: anObject on: aStream 
	aStream
		cr;
		nextPut: $!!;
		cr;
		cr;
		nextPutAll: '(FileStream oldFileNamed: ''';
		print: anObject asPath;
		nextPutAll: ''') fileIn; close';
		cr! !

!MDRunScriptJob methodsFor: 'running' stamp: 'cwp 9/17/2009 22:21'!
writeLoadScript: aReference 
	aReference fileStreamDo:
		[:stream | self writeScriptOn: stream]! !

!MDRunScriptJob methodsFor: 'running' stamp: 'cwp 12/19/2008 13:26'!
writePostscriptOn: aStream
	self writePostscriptOn: aStream saving: true! !

!MDRunScriptJob methodsFor: 'running' stamp: 'cwp 12/19/2008 13:38'!
writePostscriptOn: aStream saving: aBoolean
	aStream
		cr; nextPut: $!!; cr; cr;
		nextPutAll: 'SmalltalkImage current snapshot: ';
		print: aBoolean;
		nextPutAll: ' andQuit: true.';
		cr! !

!MDRunScriptJob methodsFor: 'running' stamp: 'cwp 12/19/2008 13:39'!
writePreambleOn: aStream
	aStream 
		nextPutAll: '(SourceFiles at: 2) ifNotNil:'; cr;
		tab; nextPutAll: '[SmalltalkImage current'; cr;
		tab; tab; nextPutAll: 'closeSourceFiles;'; cr;
		tab; tab; nextPutAll: 'saveChangesInFileNamed: '''; 
		print: self changesPath;
		nextPutAll: '''].'; cr; cr;
		nextPutAll: 'SmalltalkImage current'; cr;
		tab; nextPutAll: 'changeImageNameTo: '''; 
		print: self imagePath;
		nextPutAll: ''';'; cr;
		tab; nextPutAll: 'closeSourceFiles;'; cr;
		tab; nextPutAll: 'openSourceFiles.'; cr.
! !

!MDRunScriptJob methodsFor: 'running' stamp: 'cwp 12/19/2008 13:26'!
writeScriptBodyOn: aStream 
	scripts do: 
		[ :ea | 
		self 
			writeFilein: ea
			on: aStream ]! !

!MDRunScriptJob methodsFor: 'running' stamp: 'cwp 12/19/2008 13:39'!
writeScriptOn: aStream 
	self writePreambleOn: aStream.
	self writeScriptBodyOn: aStream.
	self writePostscriptOn: aStream! !

MDJob subclass: #MDWriteScriptJob
	instanceVariableNames: 'path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Build'!

!MDWriteScriptJob methodsFor: 'accessing' stamp: 'cwp 12/23/2008 12:10'!
addScriptTo: aRunScriptJob 
	aRunScriptJob addScript: self path! !

!MDWriteScriptJob methodsFor: 'accessing' stamp: 'cwp 12/20/2008 12:01'!
name
	^ self path basename! !

!MDWriteScriptJob methodsFor: 'accessing' stamp: 'cwp 12/19/2008 13:48'!
path
	^ path! !

!MDWriteScriptJob methodsFor: 'accessing' stamp: 'cwp 9/16/2009 23:09'!
path: anObject
	path := anObject! !

!MDWriteScriptJob methodsFor: 'accessing' stamp: 'cwp 9/20/2009 00:24'!
requires
	^ Set with: self path parent! !

!MDWriteScriptJob methodsFor: 'running' stamp: 'cwp 9/19/2009 23:02'!
run
	path fileStreamDo: [:stream | self writeOn: stream]! !

!MDWriteScriptJob methodsFor: 'dependencies' stamp: 'cwp 12/19/2008 13:48'!
supplies
	^ Set with: self path! !

!MDWriteScriptJob methodsFor: 'running' stamp: 'cwp 12/19/2008 13:48'!
writeOn: stream
	self subclassResponsibility! !

MDWriteScriptJob subclass: #MDWriteTestScriptJob
	instanceVariableNames: 'packages report'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Build'!

!MDWriteTestScriptJob methodsFor: 'running' stamp: 'cwp 12/22/2008 18:34'!
fileOutRunnerOn: aStream 
	MDTestRunner fileOutOn: aStream.
	aStream cr.! !

!MDWriteTestScriptJob methodsFor: 'accessing' stamp: 'cwp 12/20/2008 00:39'!
packages
	^ packages! !

!MDWriteTestScriptJob methodsFor: 'accessing' stamp: 'cwp 12/20/2008 00:39'!
packages: anObject
	packages := anObject! !

!MDWriteTestScriptJob methodsFor: 'accessing' stamp: 'cwp 12/20/2008 01:19'!
report
	^ report! !

!MDWriteTestScriptJob methodsFor: 'accessing' stamp: 'cwp 12/20/2008 01:19'!
report: anObject
	report := anObject! !

!MDWriteTestScriptJob methodsFor: 'running' stamp: 'cwp 12/20/2008 00:43'!
writeOn: aStream 
	self fileOutRunnerOn: aStream.
	self writeRunCommandOn: aStream! !

!MDWriteTestScriptJob methodsFor: 'running' stamp: 'cwp 9/20/2009 00:18'!
writeRunCommandOn: aStream 
	aStream
		nextPutAll: 'Utilities setAuthorInitials: ''mason''.';
		cr; cr.
	aStream
		nextPutAll: 'MDTestRunner new';
		cr;
		tab;
		nextPutAll: 'path: ''';
		print: self report path;
		nextPutAll: ''';';
		cr.
	self packages do: 
		[ :pkg | 
		aStream
			tab;
			nextPutAll: 'addPackageNamed: ';
			print: pkg;
			nextPut: $;;
			cr ].
	aStream
		tab;
		nextPutAll: 'run';
		cr! !

Object subclass: #MDProduct
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Products'!

!MDProduct class methodsFor: 'as yet unclassified' stamp: 'cwp 7/12/2009 04:53'!
build
	^ self new build! !

!MDProduct class methodsFor: 'as yet unclassified' stamp: 'cwp 7/12/2009 05:25'!
specifyOn: targets
	^ self new specifyOn: targets! !

!MDProduct methodsFor: 'building' stamp: 'cwp 7/14/2009 00:40'!
build
	self target createBuild run! !

!MDProduct methodsFor: 'specifying' stamp: 'cwp 7/12/2009 04:44'!
specifyOn: targets! !

!MDProduct methodsFor: 'building' stamp: 'cwp 7/12/2009 04:44'!
target
	| canvas |
	canvas := MDTargetCanvas new.
	self specifyOn: canvas.
	^ canvas current! !

Object subclass: #MDTarget
	instanceVariableNames: 'parent children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

MDTarget subclass: #MDDirectoryTarget
	instanceVariableNames: 'path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

!MDDirectoryTarget methodsFor: 'accessing' stamp: 'cwp 9/20/2009 09:58'!
addAllJobsTo: context
	self addJobsTo: context.
	context 
		directory: path
		do: [children do: [:ea | ea addAllJobsTo: context]]! !

!MDDirectoryTarget methodsFor: 'build' stamp: 'cwp 9/20/2009 09:33'!
addJobsTo: context
	MDCreateDirectoryJob new
		path: (context resolve: path);
		addToBuild: context! !

!MDDirectoryTarget methodsFor: 'testing' stamp: 'cwp 1/10/2009 23:28'!
isDirectory	
	^ true! !

!MDDirectoryTarget methodsFor: 'accessing' stamp: 'cwp 12/15/2008 14:20'!
path
	^ path! !

!MDDirectoryTarget methodsFor: 'accessing' stamp: 'cwp 12/15/2008 12:40'!
path: anObject
	path := anObject asPath! !

!MDDirectoryTarget methodsFor: 'accessing' stamp: 'cwp 1/10/2009 23:55'!
pathInContext
	^ parent isDirectory
		ifTrue: [parent pathInContext resolve: path]
		ifFalse: [path]! !

MDTarget subclass: #MDFileSystemTarget
	instanceVariableNames: 'jobClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

!MDFileSystemTarget methodsFor: 'accessing' stamp: 'cwp 1/10/2009 23:08'!
jobClass
	^ jobClass! !

!MDFileSystemTarget methodsFor: 'accessing' stamp: 'cwp 1/10/2009 23:08'!
jobClass: anObject
	jobClass := anObject! !

MDFileSystemTarget subclass: #MDFileTarget
	instanceVariableNames: 'path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

!MDFileTarget methodsFor: 'build' stamp: 'cwp 1/10/2009 23:35'!
addJobsTo: aBuild
	| job |
	job := MDCopyFileJob
		source: (aBuild directory resolve: path)
		dest: (aBuild directory resolve: parent pathInContext) / path basename.
	job addToBuild: aBuild! !

!MDFileTarget methodsFor: 'accessing' stamp: 'cwp 1/10/2009 23:14'!
path
	^ path! !

!MDFileTarget methodsFor: 'accessing' stamp: 'cwp 1/10/2009 23:14'!
path: anObject
	path := anObject asPath! !

MDFileSystemTarget subclass: #MDFilesTarget
	instanceVariableNames: 'directoryPath pattern'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

!MDFilesTarget methodsFor: 'as yet unclassified' stamp: 'cwp 1/10/2009 23:36'!
addJobsTo: aBuild 
	| sourcePath sourceDir source dest destPath |
	sourcePath := aBuild directory resolve: directoryPath.
	destPath := aBuild directory resolve: parent pathInContext.
	sourceDir := FileDirectory on: sourcePath asString.
	(sourceDir fileNamesMatching: pattern) do: 
		[ :name | 
		source := sourcePath / name.
		dest := destPath / name.
		aBuild addJob: (jobClass source: source dest: dest) ]! !

!MDFilesTarget methodsFor: 'accessing' stamp: 'cwp 1/9/2009 18:13'!
directoryPath
	^ directoryPath! !

!MDFilesTarget methodsFor: 'accessing' stamp: 'cwp 1/9/2009 18:17'!
directoryPath: anObject
	directoryPath := anObject asPath! !

!MDFilesTarget methodsFor: 'accessing' stamp: 'cwp 1/9/2009 18:13'!
pattern
	^ pattern! !

!MDFilesTarget methodsFor: 'accessing' stamp: 'cwp 1/9/2009 18:13'!
pattern: anObject
	pattern := anObject! !

MDTarget subclass: #MDImageContentTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

!MDImageContentTarget methodsFor: 'jobs' stamp: 'cwp 12/23/2008 14:05'!
queueJobsIn: jobs build: aBuild name: aString
	self subclassResponsibility! !

!MDImageContentTarget methodsFor: 'jobs' stamp: 'cwp 6/1/2009 21:35'!
scriptPathIn: aPath basename: aString ordinal: anInteger
	| name |
	name := String streamContents:
		[:out |
		out nextPutAll: (aString copyUpToLast: FSPath extensionDelimiter).
		out nextPut: $-.
		out print: anInteger.
		out nextPutAll: '.st'].
	^ aPath / name! !

MDImageContentTarget subclass: #MDScriptTarget
	instanceVariableNames: 'path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

!MDScriptTarget methodsFor: 'accessing' stamp: 'cwp 6/1/2009 21:35'!
addScriptTo: aRunScriptJob 
	aRunScriptJob addScript: (FSPath resolve: self path)! !

!MDScriptTarget methodsFor: 'accessing' stamp: 'cwp 12/23/2008 14:01'!
addToBuild: aBuild
	! !

!MDScriptTarget methodsFor: 'accessing' stamp: 'cwp 12/23/2008 14:03'!
path
	^ path! !

!MDScriptTarget methodsFor: 'accessing' stamp: 'cwp 12/23/2008 14:03'!
path: anObject
	path := anObject asPath! !

!MDScriptTarget methodsFor: 'accessing' stamp: 'cwp 12/23/2008 14:01'!
queueJobsIn: queue build: aBuild name: aString
	queue add: self! !

MDTarget subclass: #MDImageTarget
	instanceVariableNames: 'name baseImage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

!MDImageTarget methodsFor: 'as yet unclassified' stamp: 'cwp 9/17/2009 09:42'!
addAllJobsTo: context 
	| run |
	context image: self name do: 
		[run := MDRunScriptJob new
			baseImagePath: (context resolve: self baseImagePath);
			targetImagePath: (context resolve: self targetImagePath);
			loadScript: (context temp: name), 'st';
			yourself.
		children do: 
			[:target |
			target 
				addAllJobsTo: context 
				scriptsDo: [:script | run addScript: script]].
		run addToBuild: context]! !

!MDImageTarget methodsFor: 'as yet unclassified' stamp: 'cwp 12/20/2008 12:05'!
baseImage: anObject
	baseImage := anObject! !

!MDImageTarget methodsFor: 'as yet unclassified' stamp: 'cwp 9/20/2009 12:09'!
baseImagePath
	^ baseImage asPath, 'image'! !

!MDImageTarget methodsFor: 'as yet unclassified' stamp: 'cwp 12/14/2008 22:15'!
name
	^ name! !

!MDImageTarget methodsFor: 'as yet unclassified' stamp: 'cwp 12/20/2008 10:28'!
name: aString
	name := aString! !

!MDImageTarget methodsFor: 'as yet unclassified' stamp: 'cwp 12/20/2008 10:28'!
targetImagePath
	^ (parent path / name) assureExtension: 'image'! !

MDTarget subclass: #MDRootTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

!MDTarget class methodsFor: 'as yet unclassified' stamp: 'cwp 1/3/2009 21:58'!
with: aBlock 
	| target |
	target := self new.
	aBlock value: target.
	^ target! !

!MDTarget methodsFor: 'build' stamp: 'cwp 12/14/2008 23:18'!
addAllJobsTo: aBuild
	self addJobsTo: aBuild.
	children do: [:ea | ea addAllJobsTo: aBuild].! !

!MDTarget methodsFor: 'accessing' stamp: 'cwp 1/3/2009 22:30'!
addChild: aTarget 
	^ self addChild: aTarget with: [:target]! !

!MDTarget methodsFor: 'accessing' stamp: 'cwp 1/3/2009 22:30'!
addChild: aTarget with: aBlock
	aTarget parent: self.
	children add: aTarget.
	aBlock value: aTarget.
	^ aTarget! !

!MDTarget methodsFor: 'build' stamp: 'cwp 12/14/2008 23:18'!
addJobsTo: aBuild
	! !

!MDTarget methodsFor: 'build' stamp: 'cwp 9/16/2009 22:48'!
createBuild
	^ MDBuildContext new
		getJobsFrom: self;
		build! !

!MDTarget methodsFor: 'initialize-release' stamp: 'cwp 12/14/2008 22:05'!
initialize
	children := OrderedCollection new.! !

!MDTarget methodsFor: 'testing' stamp: 'cwp 1/10/2009 23:28'!
isDirectory
	^ false! !

!MDTarget methodsFor: 'accessing' stamp: 'cwp 12/14/2008 19:55'!
parent
	^ parent! !

!MDTarget methodsFor: 'accessing' stamp: 'cwp 1/3/2009 22:09'!
parent: aTarget
	parent := aTarget.
	! !

!MDTarget methodsFor: 'accessing' stamp: 'cwp 6/1/2009 21:36'!
path
	^ FSPath new! !

!MDTarget methodsFor: 'targets' stamp: 'cwp 7/12/2009 00:29'!
root
	^ parent root! !

!MDTarget methodsFor: 'targets' stamp: 'cwp 7/12/2009 00:34'!
with: aBlock
	^ self root current: self do: aBlock! !

MDTarget subclass: #MDTestRunTarget
	instanceVariableNames: 'name image'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

!MDTestRunTarget methodsFor: 'as yet unclassified' stamp: 'cwp 9/20/2009 00:04'!
addJobsTo: context 
	context 
		image: self name
		do: 
			[ | script |
			script := context temp: self scriptName.
			MDWriteTestScriptJob new
				path: script;
				report: (context resolve: name), 'txt';
				packages: self packageNames;
				addToBuild: context.
			MDRunScriptJob new
				baseImagePath: (context resolve: self baseImagePath);
				targetImagePath: (context resolve: self targetImagePath);
				loadScript: (context temp: name) , 'st';
				addScript: script;
				addToBuild: context ]! !

!MDTestRunTarget methodsFor: 'as yet unclassified' stamp: 'cwp 9/20/2009 00:05'!
baseImagePath
	^ (parent path resolve: image) , 'image'! !

!MDTestRunTarget methodsFor: 'as yet unclassified' stamp: 'cwp 1/3/2009 22:37'!
imagePath: aString
	image := aString! !

!MDTestRunTarget methodsFor: 'as yet unclassified' stamp: 'cwp 9/19/2009 22:37'!
name
	^ name! !

!MDTestRunTarget methodsFor: 'as yet unclassified' stamp: 'cwp 12/20/2008 00:49'!
name: aString
	name := aString! !

!MDTestRunTarget methodsFor: 'as yet unclassified' stamp: 'cwp 12/20/2008 01:14'!
packageNames
	^ children collect: [:ea | ea package]! !

!MDTestRunTarget methodsFor: 'as yet unclassified' stamp: 'cwp 9/20/2009 00:30'!
scriptName
	^ 'run.st'! !

!MDTestRunTarget methodsFor: 'as yet unclassified' stamp: 'cwp 9/20/2009 00:05'!
targetImagePath
	^ (parent path resolve: name) , 'image'! !

MDTarget subclass: #MDTestSuiteTarget
	instanceVariableNames: 'package'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

!MDTestSuiteTarget methodsFor: 'as yet unclassified' stamp: 'cwp 12/20/2008 01:07'!
package
	^ package! !

!MDTestSuiteTarget methodsFor: 'as yet unclassified' stamp: 'cwp 12/20/2008 00:51'!
package: aString
	package := aString! !

MDTarget subclass: #MDZipTarget
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

!MDZipTarget methodsFor: 'as yet unclassified' stamp: 'cwp 9/21/2009 16:58'!
addAllJobsTo: context 
	| reference filesystem requires |
	reference := context resolve: name.
	filesystem := FSZipFilesystem atReference: reference.
	self addOpenJobFor: filesystem to: context.
	requires := Set new.
	context 
		filesystem: filesystem
		named:  name
		do: [children do: [:ea | ea addAllJobsTo: context]]
		notify: [:ref | (filesystem root contains: ref) ifTrue: [requires add: ref]].
	self addCloseJobFor: filesystem to: context requiring: requires! !

!MDZipTarget methodsFor: 'as yet unclassified' stamp: 'cwp 9/21/2009 17:36'!
addCloseJobFor: aFilesystem to: context requiring: requires
	MDCloseFilesystemJob new
		filesystem: aFilesystem;
		requires: requires;
		supplies: (Set with: aFilesystem reference);
		addToBuild: context! !

!MDZipTarget methodsFor: 'as yet unclassified' stamp: 'cwp 9/21/2009 17:32'!
addOpenJobFor: aFilesystem to: context
	MDOpenFilesystemJob new
		filesystem: aFilesystem;
		requires: (Set with: aFilesystem reference parent);
		supplies: (Set with: aFilesystem root);
		addToBuild: context! !

!MDZipTarget methodsFor: 'as yet unclassified' stamp: 'cwp 9/20/2009 12:47'!
name
	^ name! !

!MDZipTarget methodsFor: 'as yet unclassified' stamp: 'cwp 9/20/2009 12:47'!
name: aString
	name := aString! !

Object subclass: #MDTargetCanvas
	instanceVariableNames: 'current'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Targets'!

!MDTargetCanvas methodsFor: 'accessing' stamp: 'cwp 7/12/2009 00:42'!
addChild: aTarget 
	current addChild: aTarget.
	^ aTarget! !

!MDTargetCanvas methodsFor: 'accessing' stamp: 'cwp 7/12/2009 00:42'!
addChild: aTarget with: aBlock
	current addChild: aTarget.
	self current: aTarget do: aBlock.
	^ aTarget! !

!MDTargetCanvas methodsFor: 'accessing' stamp: 'cwp 7/12/2009 00:35'!
current
	^ current! !

!MDTargetCanvas methodsFor: 'accessing' stamp: 'cwp 7/12/2009 00:34'!
current: aTarget do: aBlock
	| previous |
	previous := current.
	current := aTarget.
	aBlock ensure: [current := previous]! !

!MDTargetCanvas methodsFor: 'targets' stamp: 'cwp 7/12/2009 00:43'!
directory
	^ self addChild: MDDirectoryTarget new 
		! !

!MDTargetCanvas methodsFor: 'targets' stamp: 'cwp 7/12/2009 00:43'!
file
	^ self addChild: MDFileTarget new! !

!MDTargetCanvas methodsFor: 'targets' stamp: 'cwp 7/12/2009 00:43'!
files
	^ self addChild: MDFilesTarget new! !

!MDTargetCanvas methodsFor: 'targets' stamp: 'cwp 7/12/2009 00:43'!
image
	^ self addChild: MDImageTarget new
		! !

!MDTargetCanvas methodsFor: 'initialize-release' stamp: 'cwp 7/12/2009 00:35'!
initialize
	
	current := MDRootTarget new.
	current parent: self.
	! !

!MDTargetCanvas methodsFor: 'accessing' stamp: 'cwp 7/12/2009 00:14'!
root
	^ self! !

!MDTargetCanvas methodsFor: 'targets' stamp: 'cwp 7/12/2009 00:44'!
script
	^ self addChild: MDScriptTarget new
		! !

!MDTargetCanvas methodsFor: 'accessing' stamp: 'cwp 7/12/2009 21:39'!
specify: anObject
	anObject specifyOn: self! !

!MDTargetCanvas methodsFor: 'targets' stamp: 'cwp 7/12/2009 00:44'!
suite
	^ self addChild: MDTestSuiteTarget new
		! !

!MDTargetCanvas methodsFor: 'targets' stamp: 'cwp 7/12/2009 00:44'!
tests
	^ self addChild: MDTestRunTarget new
		! !

!MDTargetCanvas methodsFor: 'targets' stamp: 'cwp 9/20/2009 12:47'!
zip
	^ self addChild: MDZipTarget new 
		! !

Object variableSubclass: #MDTestRunner
	instanceVariableNames: 'packages path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mason-Core-Utilities'!

!MDTestRunner methodsFor: 'as yet unclassified' stamp: 'cwp 12/19/2008 23:46'!
addPackageNamed: aString
	packages add: (PackageInfo named: aString)! !

!MDTestRunner methodsFor: 'as yet unclassified' stamp: 'cwp 12/19/2008 23:56'!
buildSuite
	| suite all |
	suite := TestSuite new.
	all := TestCase buildSuite.
	all tests do: 
		[:test |
		(packages anySatisfy: [:pkg | pkg includesMethod: test selector ofClass: test class])
			ifTrue: [suite addTest: test]].
	^ suite! !

!MDTestRunner methodsFor: 'as yet unclassified' stamp: 'cwp 12/19/2008 23:46'!
initialize
	packages := Set new! !

!MDTestRunner methodsFor: 'as yet unclassified' stamp: 'cwp 12/20/2008 11:06'!
path: aString
	path := aString! !

!MDTestRunner methodsFor: 'as yet unclassified' stamp: 'cwp 12/20/2008 12:18'!
run
	| result initials |
	initials := Utilities authorInitialsPerSe.
	[Utilities setAuthorInitials: 'test runner'.
	result := self buildSuite run.
	self writeResult: result]
		ensure: [Utilities setAuthorInitials: initials].! !

!MDTestRunner methodsFor: 'as yet unclassified' stamp: 'cwp 12/20/2008 11:06'!
writeResult: aResult
	FileStream 
		forceNewFileNamed: path 
		do: [:stream | self writeResult: aResult on: stream] ! !

!MDTestRunner methodsFor: 'as yet unclassified' stamp: 'cwp 12/20/2008 22:46'!
writeResult: aResult on: aStream
	aStream
		nextPutAll: 'Test Result, ';
		nextPutAll: (TimeStamp now asString);
		cr;
		print: aResult;
		cr.
	
	self writeTests: aResult errors as: 'Errors' on: aStream.
	self writeTests: aResult failures as: 'Failures' on: aStream.! !

!MDTestRunner methodsFor: 'as yet unclassified' stamp: 'cwp 12/20/2008 22:46'!
writeTests: aCollection as: aString on: aStream
	aCollection isEmpty ifTrue: [^ self].
	aStream 
		cr;
		nextPutAll: aString;
		cr.
	aCollection do:
		[:test |
		aStream 
			nextPutAll: test class name;
			nextPutAll: '>>';
			nextPutAll: test selector;
			cr]! !
